%% Gnre les courbes I=f(U)et P=f(U) d'un module photovoltaque
% Le script ci-dessous permet de tracer les courbes I=f(U) et P=f(U) du 
% modle Simulink  SIMU_PV.mdl. Le script peut tre appel  partir du
% modle simulink SIMU_PV.mdl. Les rsultats de la simulation sont enregistrs
% dans PVDataSimulation dans le rpertoire DataFiles. Si le rpertoire n'  
% existe pas il sera automatiquement cr. Pour plus de lisibilit sur
% les figures j'ai volontairement limit le nombre d'essais  6.
%
% Ci-dessous, les paramtres du module correspondent au BP 380 de BP SOLAR
% (borne escamotable solaire dans l'acadmie de Lille).
% En fonction du module que vous utilisez, vous devez modifier Isc_module,
% Voc_module, et Rs_module. Modifier le modle SIMU_PV  si le nombre de
% cellules du module est diffrent de 36. Les paramtres irradiance et
% temprature du module sont directement modifiables dans simulink.
%
% Cardon Franois
% Acadmie de Lille
% STI2D option EE
% Lyce Lonard de Vinci_Calais
%
% Dernire modification en Janvier 2011 
% Version: R2010b

%% Ouvre le modle simulink
open_system('SIMU_PV')

%% Paramtres du module photovoltaque
Isc_module = 4.8;       % Courant de court-circuit du module (Ampre)
Voc_module = 22.1;      % Tension  circuit ouvert du module (Volts)
Rs_module = 0.12;       % Rsistance srie du module (Ohms)
TIPH1 = 0.001;          % First order temperature coefficient for
                        % solar generated current Iph [1/C]
TXIS1 = 3;              % Temperature exponent for diode saturation
                        % current Is1
TRS1 = 0;               % Temperature exponent for series resistance Rs

G=get_param('SIMU_PV/Irradiance','G');                                          % Irradiance W/m
T_module=get_param('SIMU_PV/Module photovoltaque 36 cellules','T_module');     % Temprature du module en C

%% Initialisation, dfinition des couleurs et de la lgende...
nbressais=6;
map = jet(nbressais);
p=isempty(findobj('name','')) ;
k=exist ('n','var') ;
if (k==0)||(n>=nbressais)||(p==1);
    n=0;
    legende_tableauH = cell(nbressais,1);
    legende_tableauB = cell(nbressais,1);
end

%% Dessine sur la figure les axes, les titres, et le nom des axes
subplot(211)
h1 = gca;
set(h1,'FontSize',9,'Box','on')
title('I=f(U)','FontWeight','bold');
xlabel('Tension (V)')
ylabel('Courant (A)')
grid('on')
hold('on')
subplot(212)
h2 = gca;
set(h2,'FontSize',9,'Box','on')
title('P=f(U)','FontWeight','bold')
xlabel('Tension (V)')
ylabel('Puissance (W)')
grid('on')
hold('on')
set(gcf,'Position',[200, 50, 800, 800])

%% Cre un rpertoire pour enregistrer les rsultats de la simulation
PVData_dir = fullfile('DataFiles','PVDataSimulation');
if(~exist(PVData_dir,'dir'))
    mkdir(PVData_dir)
end

%% Dmarre les simulations, sauvegarde les rsultats de la simulation, et trace les courbes
    n=n+1;
    sim('SIMU_PV');
    PVCourant = PVMesure.signals(1).values(:);
    PVTension = PVMesure.signals(2).values(:);
    PVPuissance = PVMesure.signals(3).values(:);
    eval(['save ' PVData_dir '\PVData_G' num2str(G) '  G PVCourant PVTension PVPuissance'])
    subplot(211)
    plot(PVTension,PVCourant,'Color',map(n,:))
    axis('tight')
    subplot(212)
    plot(PVTension,PVPuissance,'Color',map(n,:))
    axis('tight')
    legende_tableauH{n} = ['G = ' num2str(G) ' W/m^2, T = '...
        num2str(T_module) ' C'];
     legende_tableauB{n} = ['G = ' num2str(G) ' W/m^2, T = '...
        num2str(T_module) ' C'];
    
%% Ajoute la lgende sur la figure
if (n==1)
    hold(h1,'off')
    legend(h1,legende_tableauH(1),'FontSize',8,'Location','NorthEastOutside')
    hold(h2,'off')
    legend(h2,legende_tableauB(1),'FontSize',8,'Location','NorthEastOutside')
end
if (n==2)
    hold(h1,'off')
    legend(h1,legende_tableauH([1,2]),'FontSize',8,'Location','NorthEastOutside')
    hold(h2,'off')
    legend(h2,legende_tableauB([1,2]),'FontSize',8,'Location','NorthEastOutside')
end
if (n==3)
    hold(h1,'off')
    legend(h1,legende_tableauH([1,2,3]),'FontSize',8,'Location','NorthEastOutside')
    hold(h2,'off')
    legend(h2,legende_tableauB([1,2,3]),'FontSize',8,'Location','NorthEastOutside')
end
if (n==4)
    hold(h1,'off')
    legend(h1,legende_tableauH([1,2,3,4]),'FontSize',8,'Location','NorthEastOutside')
    hold(h2,'off')
    legend(h2,legende_tableauB([1,2,3,4]),'FontSize',8,'Location','NorthEastOutside')
end
if (n==5)
    hold(h1,'off')
    legend(h1,legende_tableauH([1,2,3,4,5]),'FontSize',8,'Location','NorthEastOutside')
    hold(h2,'off')
    legend(h2,legende_tableauB([1,2,3,4,5]),'FontSize',8,'Location','NorthEastOutside')
end
if (n==6)
    hold(h1,'off')
    legend(h1,legende_tableauH([1,2,3,4,5,6]),'FontSize',8,'Location','NorthEastOutside')
    hold(h2,'off')
    legend(h2,legende_tableauB([1,2,3,4,5,6]),'FontSize',8,'Location','NorthEastOutside')
end
